/*************************************************************
 * CSCI P436 - Operating Systems
 *
 * John David Ratliff jdratlif@cs.indiana.edu
 * Chris O'Donnell chodonne@cs.indiana.edu
 *
 * Project 1 - Mini http Server
**************************************************************/

#ifndef _MHTTPD_H
#define _MHTTPD_H

#include <pthread.h>

#include "socket_queue.h"

/**************************************************************/
// comment this out for real environments...
//#define DEBUG

/* Global Constants */

#define	DEFAULT_PORT	64201
#define DEFAULT_WORKERS	10

#define MHTTPD_VERSION	"1.0.0"

#define HTTP_DATE_FMT	"%a, %d %b %Y %H:%M:%S GMT"

#define ERROR -1
#define SUCCESS 1

enum TF {FALSE,TRUE};

/* Structures */

typedef struct {
	int port;		// the server port
	int workers;		// worker thread count
	char http_dir[1024];	// the http root directory
} SERVER_DATA;

/* Shared Data */

extern SOCKET_QUEUE *queue;
extern pthread_mutex_t queue_mutex;
extern pthread_cond_t queue_empty, queue_full;

/* Function prototypes */

/* mhttpd.c */

void print_version_info(void);
void print_help_info(void);
void parse_command_line(int, char **, SERVER_DATA *);
int start_server(int);
int main(int, char **);

/* parse.c */

void send_http_header(int, int, const char *, unsigned long int, const char *, time_t);
void send_error_page(int, int, const char *, const char *);
int get_next_line(const char *, char *, int);
void parse(int, const char *, const char *);

/* server.c */

void handle_connection(int, char *);
void worker(void *);
void boss(void *);

#endif

/******************************************************************************
 *  Mini http server 1.0.0
 *  Copyright (C) 2002 John David Ratliff [jdratlif@cs.indiana.edu]
 *                     Chris O'Donnell    [chodonne@cs.indiana.edu]
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation, 
 *  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

