/*************************************************************
 * CSCI P436 - Operating Systems
 *
 * John David Ratliff jdratlif@cs.indiana.edu
 * Chris O'Donnell chodonne@cs.indiana.edu
 *
 * Project 1 - Mini http Server
**************************************************************/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <pthread.h>

#include <sys/socket.h>
#include <netinet/in.h>

#include "mhttpd.h"

// reads a request from the socket and responds in the HTTP protocol
void handle_connection(int sockfd, char *http_dir) {
	char buffer[1024];

	// clear our receive buffer	
	memset(buffer,0,1024);
	
	// read the request from the socket
	if (recv(sockfd, buffer, 1023, 0) > 0) {
#ifdef DEBUG
		printf("Client said [%s]\n",buffer);
#endif
		
		// parse and respond to the HTTP request
		parse(sockfd,http_dir,buffer);
	}
	
	// close the socket when done
	close(sockfd);
}

// monitor the socket queue and jump on the first entry we find
void worker(void *http_dir) {
	int sockfd;
	
	// work till we die	
	for (;;) {
		// lock the mutex
		pthread_mutex_lock(&queue_mutex);
		
		// wait until our signal comes in
		while (socketqueue_is_empty(queue)) {
#ifdef DEBUG
			printf("worker: waiting on socketqueue...\n");
#endif

			// wait until we get something in the queue
			pthread_cond_wait(&queue_empty,&queue_mutex);
		}
		
#ifdef DEBUG
		printf("worker: handling socket connection...\n");
#endif

		// grab the client connection socket
		sockfd = socketqueue_dequeue(queue);
		
		// signal the queue no longer full
		pthread_cond_signal(&queue_full);
		
		// unlock the mutex
		pthread_mutex_unlock(&queue_mutex);

		// handle the connection request
		handle_connection(sockfd, (char *)http_dir);
	}
}

// waits for socket connections and add them to the socket queue
void boss(void *server_socket) {
	struct sockaddr_in remoteAddress;
	int remoteAddressSize = sizeof(struct sockaddr_in);
	int remoteSocket;
	int sockfd = *(int *)server_socket;
	
	// listen for connections for...ever (or until we get a shutdown message)
	for (;;) {
		// accept the remote connection
		remoteSocket = accept(sockfd, (struct sockaddr *)&remoteAddress, &remoteAddressSize);
		
#ifdef DEBUG
		printf("boss: accepted remote connection. adding to socketqueue...\n");
#endif
		
		// acquire the queue mutex
		pthread_mutex_lock(&queue_mutex);
		
		// wait if the queue is full
		while (socketqueue_is_full(queue)) {
			pthread_cond_wait(&queue_full,&queue_mutex);
		}
		
		// enqueue the socket
		socketqueue_enqueue(queue,remoteSocket);
		
		// signal that the queue is no longer empty
		pthread_cond_signal(&queue_empty);
		
		// release the queue mutex
		pthread_mutex_unlock(&queue_mutex);
	}
}

/******************************************************************************
 *  Mini http server 1.0.0
 *  Copyright (C) 2002 John David Ratliff [jdratlif@cs.indiana.edu]
 *                     Chris O'Donnell    [chodonne@cs.indiana.edu]
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation, 
 *  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

