/*************************************************************
 * CSCI P436 - Operating Systems
 *
 * John David Ratliff jdratlif@cs.indiana.edu
 * Chris O'Donnell chodonne@cs.indiana.edu
 *
 * Project 1 - Mini http Server
**************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "mhttpd.h"
#include "socket_queue.h"

// create a new socketqueue of default_size
// return the socketqueue if successful, NULL otherwise
SOCKET_QUEUE *socketqueue_create(void) {
	SOCKET_QUEUE *queue;

	// allocate memory for the socketqueue structure
	if ((queue = malloc(sizeof(SOCKET_QUEUE))) == NULL) {
		fprintf(stderr,"Error: Unable to allocate memory for our queue!\n");
		return NULL;
	}
	
	// setup the queue defaults
	queue->size = DEFAULT_QUEUE_SIZE;
	queue->length = 0;
	queue->in = 0;
	queue->out = 0;
	
	// allocate memory for the internal queue structure
	if ((queue->queue = calloc(queue->size,sizeof(int))) == NULL) {
		fprintf(stderr,"Error: Unable to allocate memory for our queue data!\n");
		free(queue);
		return NULL;
	}
	
	// return the socket queue
	return queue;
}

// delete a socketqueue structure and free all associated memory
void socketqueue_delete(SOCKET_QUEUE *queue) {
	free(queue->queue);
	free(queue);
}

// test if this socketqueue is empty
short int socketqueue_is_empty(SOCKET_QUEUE *queue) {
	if (queue->length == 0) {
		return TRUE;
	}
	
	return FALSE;
}

// test if this socketqueue is full
short int socketqueue_is_full(SOCKET_QUEUE *queue) {
	if (queue->length == queue->size) {
		return TRUE;
	}
	
	return FALSE;
}

// add an element to the queue
void socketqueue_enqueue(SOCKET_QUEUE *queue, int sockfd) {
	// add the socket to the queue
	queue->queue[queue->in] = sockfd;
	
	// find the next position in the queue, wrapping around
	if (queue->in == queue->size) {
		queue->in = 0;
	} else {
		queue->in++;
	}
	
	// increment the queue length
	queue->length++;
}

// remove an element from the queue
int socketqueue_dequeue(SOCKET_QUEUE *queue) {
	int sockfd;

	// can't dequeue the empty queue
	if (socketqueue_is_empty(queue)) {
		return ERROR;
	}

	// get the element from the queue
	sockfd = queue->queue[queue->out];

	// update the queue pointer, wrapping around		
	if (queue->out == queue->size) {
		queue->out = 0;
	} else {
		queue->out++;
	}
		
	// decrement the queue length
	queue->length--;

	// return the element dequeued
	return sockfd;
}

/******************************************************************************
 *  Mini http server 1.0.0
 *  Copyright (C) 2002 John David Ratliff [jdratlif@cs.indiana.edu]
 *                     Chris O'Donnell    [chodonne@cs.indiana.edu]
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation, 
 *  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

